/*
 * Copyright (c) 2021, MegaEase
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package operator

import (
	installbase "github.com/megaease/easemeshctl/cmd/client/command/meshinstall/base"

	"github.com/pkg/errors"
	v1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/util/intstr"
)

func serviceSpec(ctx *installbase.StageContext) installbase.InstallFunc {
	labels := meshOperatorLabels()

	service := &v1.Service{
		ObjectMeta: metav1.ObjectMeta{
			Name:      installbase.OperatorServiceName,
			Namespace: ctx.Flags.MeshNamespace,
		},
	}
	service.Spec.Ports = []v1.ServicePort{
		{
			Name:       "https",
			Port:       8443,
			TargetPort: intstr.IntOrString{StrVal: "https"},
		},
		{
			Name:       "mutate-webhook",
			Port:       9090,
			TargetPort: intstr.IntOrString{StrVal: "mutate-webhook"},
		},
	}
	service.Spec.Selector = labels
	return func(ctx *installbase.StageContext) error {
		err := installbase.DeployService(service, ctx.Client, ctx.Flags.MeshNamespace)
		if err != nil {
			return errors.Wrapf(err, "Create operator service %s", ctx.Flags.MeshNamespace)
		}
		return err
	}
}
