/*
 * Copyright (c) 2021, MegaEase
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package resource

import (
	"github.com/megaease/easemesh-api/v2alpha1"
	"github.com/megaease/easemeshctl/cmd/client/resource/meta"
)

type (
	// Resilience describes resilience resource of the EaseMesh
	Resilience struct {
		meta.MeshResource `yaml:",inline"`
		Spec              *v2alpha1.Resilience `yaml:"spec" jsonschema:"required"`
	}
)

// ToV2Alpha1 converts a Resilience resource to v2alpha1.Resilience
func (r *Resilience) ToV2Alpha1() *v2alpha1.Resilience {
	return r.Spec
}

// ToResilience converts a v2alpha1.Resilience resource to a Resilience resource
func ToResilience(name string, resilience *v2alpha1.Resilience) *Resilience {
	result := &Resilience{
		Spec: &v2alpha1.Resilience{},
	}
	result.MeshResource = NewResilienceResource(DefaultAPIVersion, name)
	result.Spec.RateLimiter = resilience.RateLimiter
	result.Spec.Retry = resilience.Retry
	result.Spec.CircuitBreaker = resilience.CircuitBreaker
	result.Spec.TimeLimiter = resilience.TimeLimiter
	result.Spec.FailureCodes = resilience.FailureCodes
	return result
}
